<?php
/**
 * @version		$Id: sh_scheme.class.php,v 1.7 2010-09-02 16:17:56 juanca Exp $
 * @package		vpl. vpl Syntaxhighlighter for Scheme language
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

require_once dirname(__FILE__).'/sh_base.class.php';

class vpl_sh_scheme extends vpl_sh_base{

	function __construct(){
		parent::__construct();
		//TODO need more reserved and functions
		$list = array('define', 'if', 'cond', 'else',
 								'let', 'eq?', 'eqv?', 'equal?',
 								'and', 'or', 'letrec', 'let-syntax',
 								'letrec-sintax', 'begin', 'do',
								'quote', '+', '-', '*', '/',
 								'sqrt', 'eval', 'car', 'cdr', 'list',
 								'cons', 'null?', 'list?', '=', '<>',
 								'<=', '>=', '<', '>', 'lambda',
 								'not');
		$this->reserved= array();
		foreach ($list as $word) {
			$this->reserved[$word]=1;
		}
	}
	function is_previous_open_parenthesis(& $string, $pos){
		for( ;$pos >= 0;$pos--){
			$char = $string[$pos];
			if($char=='('){
				return true;
			}
			if($char != ' ' && $char != self::TAB && $char != self::LF && $char != self::CR){
				return false;
			}
		}
		return false;
	}
	const in_regular=0;
	const in_string=1;
	const in_comment=4;

	function print_file($filename, $filedata, $showln=true){
		$this->begin($filename,$showln);
		$state = self::in_regular;
		$pending='';
		$previous_is_open_parenthesis = false;
		$l = strlen($filedata);
		if($l){
			$this->show_line_number();
		}
		$current='';
		$pospendig=0;
		for($i=0;$i<$l;$i++){
			$previous=$current;
			$current=$filedata[$i];
			if($i < ($l-1)) {
				$next = $filedata[$i+1];
			}else{
				$next ='';
			}
			if($current == self::CR){
				if($next == self::LF) {
					continue;
				}else{
					$current = self::LF;
				}
			}
			switch($state){

				case self::in_comment:{
					if($current==self::LF) {
						$this->endTag();
						$state = self::in_regular;
					}
					$this->show_text($current);
					if($current==self::LF) {
						$this->show_line_number();
					}
					break;
				}
				case self::in_string:{
					$this->show_text($current);
					if($current=='"' && $previous!="\\") {
						$this->endTag();
						$state = self::in_regular;
					}
					if($current==self::LF) {
						$this->show_line_number();
					}
					break;
				}
				case self::in_regular:{
					if(($current != ' ') && ($current != '(')&& ($current!=')') 
					&& ($current != ';')&& ($current != '"') && ($current!=self::LF) && ($current!=self::TAB)) {
						if($pending == ''){
							$pospendig=$i;
						}
						$pending .= $current;
					}else{
						if(strlen($pending)){
							if($this->is_previous_open_parenthesis($filedata, $pospendig-1)){
								if(array_key_exists($pending, $this->reserved)){
									$class=self::c_reserved;
								}else{
									$class=self::c_variable;
								}
								$this->initTag($class);
								$this->show_pending($pending);
								$this->endTag();
							}else{
								if($pending == "#t" || $pending == "#f"){
									$this->initTag(self::c_reserved);
									$this->show_pending($pending);
									$this->endTag();
								}else{
									$this->show_pending($pending);
								}
							}
						}
						if($current == ';'){
							$state = self::in_comment;
							$this->initTag(self::c_comment);
						} elseif($current == '"')	{
							$state = self::in_string;
							$this->initTag(self::c_string);
						}
						$this->show_text($current);
						if($current == self::LF){
							$this->show_line_number();
						}
					}
					break;
				}
			}
		}
		if(strlen($pending)){
			$this->show_pending($pending);
		}
		if($state == self::in_regular)	{
			$this->endTag();
		}
		$this->end();
	}
}

?>
